/*
 * LCDControl.c
 *
 *  Created on: Jul 8, 2015
 *      Author: RK
 */

#include <SI_EFM8UB1_Register_Enums.h>		// SFR declarations
#include "USBVCPCommunication_Defs.h"


//Global variables
unsigned char LCDModeSelect;
unsigned char LCDTxState = IDLE;
xdata unsigned char  LCDDisplayData[NUM_LINES][NUM_LINE_DATA_BYTES];
unsigned char LCDCursor[2];




///////////////////////////////////////////////////////////////////////////////////////

void InitLCD()
{
	unsigned char row;
	unsigned char column;

	SPI_CS = LOW;															//make sure that the chip select signal is LOW

	for(row = 0; row < NUM_LINES; row++)
	{
		for(column = 0; column < NUM_LINE_DATA_BYTES; column++)
		{
			LCDDisplayData[row][column] = 0xFF;
		}
	}

	LCDCursor[ROW] = 0;						//initialize the cursor to the top left
	LCDCursor[COL] = 0;

	LCDAllClear();										//first clear the LCD to all white pixels
}

void UpdateAllLCDLines()
{
	SPI_CS = HIGH;																//set chip select HIGH to enable data transfer to the LCD

	SFRPAGE = TIMER4_PAGE; TMR4L = 0; while(TMR4L < 30);				//this ensures adequate delay from driving chip select high to the start of data transfer (tsSCS in the LCD data sheet)

	LCDTxState = DATA_UPDATE_MODE_SENT;								//this tells the ISR to send the address byte after this mode select byte is sent

	SFRPAGE = SPI0_PAGE;
	SPI0DAT = 0x00 | BIT7;														//this initiates transfer of the first byte (the mode select byte)
																										// 0x00 | BIT7 corresponds to "data update mode"

	/*when transfer of this byte is complete, the SPI interrupt will fire, and the rest
	of the transfer is handled in the ISR*/
}


void LCDAllClear()
{
	unsigned char row, column;

	for(row = 0; row < NUM_LINES; row++)
	{
		for(column = 0; column < NUM_LINE_DATA_BYTES; column++)
		{
			LCDDisplayData[row][column] = 0xFF;
		}
	}

	LCDCursor[ROW] = 0;
	LCDCursor[COL] = 0;

	SPI_CS = HIGH;																//Set chip select HIGH to enable data transfer to the LCD

	SFRPAGE = TIMER4_PAGE; TMR4L = 0; while(TMR4L < 30);			//this ensures adequate delay from driving chip select high to the start of data transfer (tsSCS in the LCD data sheet)

	LCDTxState = ALL_CLEAR_SENT;										//this tells the ISR that this transfer sequence is an all clear command, not a transmission of pixel data

	SFRPAGE = SPI0_PAGE;
	SPI0DAT = 0x00 | BIT5;														//this initiates transfer of the first byte (the mode select byte)
																										// 0x00 | BIT5 corresponds to "all clear mode"

	/*when transfer of this byte is complete, the SPI interrupt will fire, and the rest
	of the transfer is handled in the ISR*/
}

void InsertCharacter(unsigned char *LCD_Character)			//the input to this function is a pointer to a pixel data array
{
	unsigned char n;
	unsigned char row_pixel;
	unsigned char column_byte;

	row_pixel = LCDCursor[ROW];							//rows are handled on a pixel-by-pixel basis
	column_byte = LCDCursor[COL];						//columns are handled on a byte-by-byte basis, because one character width is 8 bits

	/*each byte from the pixel data array is copied to LCDDisplayData,
	starting with the current cursor position*/
	for(n = 0; n < CHAR_HEIGHT; n++)
	{
		LCDDisplayData[row_pixel][column_byte] = *LCD_Character;
		LCD_Character++;							//point to the next byte in the array
		row_pixel++;									//the data in the next byte corresponds to the pixel data for the next line
	}

	LCDCursor[COL]++;						//move the cursor one character width to the right

	/*if the cursor has reached the end of the line,
	 return the cursor to the far left and move it
	 down by one character height*/
	if(LCDCursor[COL] == NUM_LINE_DATA_BYTES)
	{
		LCDCursor[COL] = 0;
		LCDCursor[ROW] = LCDCursor[ROW] + CHAR_HEIGHT;
		if(LCDCursor[ROW] == NUM_LINES)		//if the cursor has reached the end of the display area,
			LCDCursor[ROW] = 0;									//return the cursor to the top line
	}

	//wait until the SPI is idle before initiating a new transfer
	while(LCDTxState != IDLE);

	UpdateAllLCDLines();
}
